var EXPORTED_SYMBOLS = ["Branding"];

const {
	classes: Cc,
	interfaces: Ci,
	results: Cr,
	utils: Cu
} = Components;

Cu.import("chrome://smartsearchbox/content/modules/registry.jsm");

var Branding = {
    
    getConfigParam: function (paramName) {
        if (this._configParams == null) {
            this._readConfig();
        }
        return this._configParams[paramName];
    
    },
    getHardwareId: function() {
        try {

            function generateGuid(hasBrackets) {
                var uuidGenerator = Cc["@mozilla.org/uuid-generator;1"].getService(Ci.nsIUUIDGenerator);
                var uuid = uuidGenerator.generateUUID().toString();
                uuid = uuid.replace(/-/g, "").toUpperCase();
                return hasBrackets ? uuid : uuid.substring(1, uuid.length - 1);
            };    

            let id = Registry.readRegistryStringValue(Registry.HKLM, "SOFTWARE\\Think Tank Labs", "guid");
            if (id == null)
                id = Registry.readRegistryStringValue(Registry.HKCU, "SOFTWARE\\Think Tank Labs", "guid");
            if (id != null) {
                id = id.replace(/-/g, "");
                id = id.replace("{", "");
                id = id.replace("}", "");
                return id;
            }

            id = generateGuid(false);
            if (!Registry.writeRegistryStringValue(Registry.HKCU, "SOFTWARE\\Think Tank Labs", "guid", id))
                return;

            return id;

        } catch(e) {  Cu.reportError(e); return "";}
    },

    getChromeContents: function(aURL) {
        var ioService = Components.classes["@mozilla.org/network/io-service;1"]
        .getService(Components.interfaces.nsIIOService);
        var scriptableStream = Components
        .classes["@mozilla.org/scriptableinputstream;1"]
        .getService(Components.interfaces.nsIScriptableInputStream);

        var channel = ioService.newChannel(aURL, null, null);
        var input = channel.open();
        scriptableStream.init(input);
        var str = scriptableStream.read(input.available());
        scriptableStream.close();
        input.close();
        return str;
    },
    
    _readBinary: function (aURL) {
        var ioService = Components.classes["@mozilla.org/network/io-service;1"]
        .getService(Components.interfaces.nsIIOService);
        var scriptableStream = Components
        .classes["@mozilla.org/scriptableinputstream;1"]
        .getService(Components.interfaces.nsIScriptableInputStream);

        var channel = ioService.newChannel(aURL, null, null);
        var input = channel.open();

        var bstream = Components.classes["@mozilla.org/binaryinputstream;1"].
                      createInstance(Components.interfaces.nsIBinaryInputStream);
        bstream.setInputStream(input);

        return bstream.readBytes(bstream.available());    
    },
    
    _readConfig: function() {
       let configContent = this.getChromeContents("chrome://smartsearchbox/content/config.xml");
       let oParser = Cc["@mozilla.org/xmlextras/domparser;1"].createInstance(Ci.nsIDOMParser);
       let xml = oParser.parseFromString(configContent, "text/xml");
       this._configParams = {};
       let rootNode = xml.getElementsByTagName("root")[0];
       for (let i = 1; i < rootNode.childNodes.length; i+=2) {
            let element = rootNode.childNodes[i];
            this._configParams[element.tagName] = element.childNodes[0].nodeValue;
       }
    },
    
    getSuggestIconDataURL: function () {
        let iconpath = this.getConfigParam("searchiconpath");
        if (iconpath == null)
            return "";
            
        let iconContent = this._readBinary("chrome://smartsearchbox/content/" + iconpath);
        return "data:image/png;base64," + btoa(iconContent);
    },
    
    getSuggestSubmissionFor: function(terms) {

        let searchURL = this.getConfigParam("suggestionsUrl");
        
        if (searchURL == null)
            return "";
        
        return searchURL.replace("{searchTerms}", terms);
    }

};
