(function () {
    function Core () {
        
        var $ = this;
        
        const UConverter = Cc["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Ci.nsIScriptableUnicodeConverter);
        UConverter.charset = "UTF-8";
        
        $.OnGetSuggest = function (sender, apiObject) {
            let text = UConverter.ConvertToUnicode(sender.getAttribute("params"));
            
            var req = new XMLHttpRequest();
            
            req.open("GET", $.Branding.getSuggestSubmissionFor(text), true);
            req.onreadystatechange = function (aEvt) {
                if (req.readyState == 4) {
                    if (req.status == 200) {
                        apiObject.sendResponse(sender, req.responseText);
                    }
                }
            };
            req.send(null); 

        };
        
        $.OnReadFile = function (sender, apiObject) {
            let url = $.getChromePath() + "/content/" + sender.getAttribute("params").replace(/\\/g, "/");
            apiObject.sendResponse(sender, $.RLLib.getContents(url));
        };
        
        $.OnAddScript = function (sender, apiObject, doc) {
            let url = $.getChromePath() + "/content/" + sender.getAttribute("params").replace(/\\/g, "/");
            $.RLLib.injectScriptToDoc(doc, url);   
            apiObject.sendResponse(sender, "true");         
        };
        
        $.OnShowLightBox = function (sender, apiObject, doc) {
            let url = $.Branding.getConfigParam("lightboxUrl");
            url = url.replace("{searchTerms}", sender.getAttribute("params"));
            url = url.replace("{documentDomain}", doc.domain);
            let scriptText = "setTimeout(function() { SeLightBox.show(\"" + url +"\"); }, 0);";
            $.RLLib.execScriptToDoc(window.content.document, scriptText);   
            apiObject.sendResponse(sender, "true");
            
        };
        
        $.OnDisableSite = function (sender, apiObject, doc) {
            
            let scriptText = "setTimeout(function() { SeLightBox.disableSite(); }, 0);";
            $.RLLib.execScriptToDoc(window.content.document, scriptText);   
            apiObject.sendResponse(sender, "true");
            
        };
        
        $.OnCloseLightBox = function (sender, apiObject, doc) {
            
            let url = sender.getAttribute("params").replace(/\\/g, "/");
            let scriptText = "setTimeout(function() {document.getElementById(\"se-lightbox\").onclick();}, 100);";
            $.RLLib.execScriptToDoc(window.content.document, scriptText);   
            apiObject.sendResponse(sender, "true");
        };
        
        $.OnSnoozeTo = function (sender, apiObject, doc) {
            
            $.PrefService.write("extensions.smartsearchbox.snooseto", sender.getAttribute("params"));
            
            let scriptText = "setTimeout(function() { SeLightBox.closeAll();}, 100);";
            $.RLLib.execScriptToDoc(window.content.document, scriptText);   
            apiObject.sendResponse(sender, "true");
        };

        $.OnGetConfigParam = function (sender, apiObject, doc) {
            apiObject.sendResponse(sender, $.Branding.getConfigParam(sender.getAttribute("params")));
        };

        $.OnIsWhitelisted = function (sender, apiObject, doc) {
            let iswhitelisted = $._whitelistExpression.test(sender.getAttribute("params"));
            
            if (!iswhitelisted) {
              apiObject.sendResponse(sender, "0");
              return true;
            }
            let text = $.Branding.getChromeContents($.getChromePath() + "/content/whitelist.xml");
            let oParser = new DOMParser();
            let xml = oParser.parseFromString(text, "text/xml");
            
            let domains = $.XMLHelper.evaluateXPath(xml, "/root/domain[@value=\"" + sender.getAttribute("params") +"\"]", xml);
            if (domains.length == 0) {
              apiObject.sendResponse(sender, "0");
              return true;
            }
            apiObject.sendResponse(sender, (new XMLSerializer()).serializeToString(domains[0]));
            return true;
        };

        $._initExpressions = function () {
            try {
                
              let text = $.Branding.getChromeContents($.getChromePath() + "/content/whitelist.xml");
              let oParser = new DOMParser();
              let xml = oParser.parseFromString(text, "text/xml");
              let domains = xml.getElementsByTagName("domain");
              
              let expression = "^(www\.)?(";
              for (let i = 0; i < domains.length; i++) {
                let domainNode = domains[i];
                if (i > 0)
                expression += "|";
               let domainExpression = domainNode.getAttribute("value");
               expression += domainExpression.replace(/\./g, "\\.").replace("*\\.", "(.*\.)?").replace("\\.*", "(\..*)?");
              }
              expression += ")$";
              
              $._whitelistExpression = new RegExp(expression);
              
            } catch(e) { Cu.reportError(e) }
         };

        $.Events = {
            load: function (e) {
                $._initExpressions();
                
                window.addEventListener("DOMContentLoaded", function(e) {
                    
                    if (Number($.PrefService.read("extensions.smartsearchbox.snooseto")) > Date.now())
                        return;
                        
                    let ClientAPI = new $.RLLib.ClientAPIInstance("search-suggestor");
                    ClientAPI.attachToDocument(e.target, function (sender) {
                        
                        if (sender.getAttribute("message") == "read-file")
                            return $.OnReadFile(sender, ClientAPI);
                        if (sender.getAttribute("message") == "get-suggest")
                            return $.OnGetSuggest(sender, ClientAPI);  
                        if (sender.getAttribute("message") == "add-script")
                            return $.OnAddScript(sender, ClientAPI, e.target);  
                        if (sender.getAttribute("message") == "show-lightbox")
                            return $.OnShowLightBox(sender, ClientAPI, e.target);  
                        if (sender.getAttribute("message") == "disable-site")
                            return $.OnDisableSite(sender, ClientAPI, e.target);  
                        if (sender.getAttribute("message") == "close-lightbox")
                            return $.OnCloseLightBox(sender, ClientAPI, e.target);  
                        if (sender.getAttribute("message") == "snooze-to")
                            return $.OnSnoozeTo(sender, ClientAPI, e.target);  
                        if (sender.getAttribute("message") == "get-config-param")
                            return $.OnGetConfigParam(sender, ClientAPI, e.target);  
                        if (sender.getAttribute("message") == "is-whitelisted")
                            return $.OnIsWhitelisted(sender, ClientAPI, e.target);  

                        Cu.reportError(sender.getAttribute("message"));
                    });

                    $.RLLib.injectScriptToDoc(e.target, "chrome://smartsearchbox/content/res/storage.js");
                    $.RLLib.injectScriptToDoc(e.target, "chrome://smartsearchbox/content/res/rlibrary.js");
                    
                    $.RLLib.injectScriptToDoc(e.target, "chrome://smartsearchbox/content/res/preprocessor.js");
                    
                }, false);
            },

            unload: function() {
                $._unloadPlugin();
            }
        };

        $.NavigateListener = {};

        $._projectName = function () {
            return "smartsearchbox";
        };

        $.getChromePath = function() { 
            return "chrome://" + $._projectName(); 
        }

        $.getResourcePath = function() { 
            return "resource://" + $._projectName(); 
        }

        $.getPrefBrunch = function() { 
            return "extensions." + $._projectName() + "."; 
        }

        $.getSkinPath = function() {
            return $.getChromePath() + "/skin/"; 
        };

        $._unloadPlugin = function UnloadPlugin() {
        };

        Cu.import($.getChromePath() + "/content/modules/rllib.js", $);
        Cu.import($.getChromePath() + "/content/modules/xmlhelper.jsm", $);
        Cu.import($.getChromePath() + "/content/modules/prefservice.jsm", $);
        Cu.import($.getChromePath() + "/content/modules/branding.jsm", $);
        

    };

    var addonCore = new Core();

    window.addEventListener("load", function(aLoadEvent) {
        aLoadEvent.currentTarget.removeEventListener("load", arguments.callee, false);

        aLoadEvent.currentTarget.addEventListener("unload", function(aUnloadEvent) {
            aUnloadEvent.currentTarget.removeEventListener("unload", arguments.callee, false);
            addonCore.Events.unload(aUnloadEvent);
        }, false);

        addonCore.Events.load(aLoadEvent)
    }, false);

})();