
function CRLLibrary(AddonBridgeName) {

    var RLLibrary = {};

    RLLibrary._instanceName = AddonBridgeName;

    RLLibrary.event = function(element, eventName, callback, flag) {
        if (!element)
            return false;
        if (element.attachEvent)
            element.attachEvent(eventName, callback);
        else if (element.addEventListener)
            element.addEventListener(eventName.replace(/^on/, ""), callback, flag == null ? false : flag);
        else
            return false;
        return true;
    };
    
    RLLibrary.IsNullOrEmpty = function(value) {
      if (value) {
        if (typeof (value) == 'string') {
          if (value.length > 0)
            return false;
        }
        if (value != null)
          return false;
      }
      return true;
    };
    
    RLLibrary.stopEvent = function(e) {
        if (e == null)
            return false;
        e.cancelBubble = true;
        if (e.stopPropagation)
            e.stopPropagation();
        if (e.preventDefault)
            e.preventDefault();
        return false;
    }

    RLLibrary.copyStylesIE = function(from, to) {
      //var str = "";
      for (i in from.currentStyle) {
        if (typeof from.currentStyle[i] == "function") {
          continue;
        }
        var attrVal = from.currentStyle.getAttribute(i);
        if (attrVal == null)
          continue;
        //str += i + ": " + attrVal + "\n";
        //alert([element.currentStyle[i], i, newInput.style])
        try {
        if (i != "className")
          to.style[i] = attrVal;
        } catch (e) {
        alert([e.message, i])
          //to.setAttribute(i, attrVal);
        }

      }
      //console.log(str)
    }

    RLLibrary.copyStyles = function(from, to) {
      //if (RLLibrary.getInternetExplorerVersion() != -1) {
        //RLLibrary.copyStylesIE(from, to);
       // return;
     // }
      var str = "";
      var currentStyles = RLLibrary.getComputedStyle(from, null)
      for (i in currentStyles) {
        var styleVal = currentStyles.getPropertyValue(i);
        if (typeof styleVal == "function" || styleVal == null || !isNaN(i)) {
          continue;
        }
        if (currentStyles.getAttribute != null) {
          styleVal = currentStyles.getAttribute(i);
          if (attrVal == null)
            continue;
        }
        str += i + ": " + styleVal + "\n";

        try {
          to.style.setProperty(i, styleVal, "");
        } catch (e) {
          to.setAttribute(i, styleVal);
        }
      }
    };

    RLLibrary.removeEvent = function(element, eventName, callback, flag) {
        if (element.detachEvent)
            element.detachEvent(eventName, callback);
        else if (element.removeEventListener)
            element.removeEventListener(eventName.replace(/^on/, ""), callback, flag == null ? false : flag);
        else
            return false;
        return true;
    };

    RLLibrary.getInternetExplorerVersion = function() {
        var rv = -1; // Return value assumes failure.
        if (navigator.appName == 'Microsoft Internet Explorer') {
            var ua = navigator.userAgent;
            var re = new RegExp("MSIE ([0-9]{1,}[\.0-9]{0,})");
            if (re.exec(ua) != null)
                rv = parseFloat(RegExp.$1);
        }
        return rv;
    };
    RLLibrary.getInnerText = function(element) {
        return element.innerText || element.textContent || element.childNodes[0].nodeValue;
    };

    RLLibrary.getZoomLevel = function() {
        var zoomLevel = 1;

        if (RLLibrary.getInternetExplorerVersion() > 6) {
            zoomLevel = screen.deviceXDPI / screen.logicalXDPI;
        } else {
            var rect = document.body.getBoundingClientRect();
            zoomLevel = Math.round((rect.right - rect.left) / document.body.clientWidth);
        }
        return zoomLevel;
    };

    RLLibrary.addStyle = function(text, id) {
        
            if (document.getElementById(id) != null)
                return false;

            var styles = document.createElement('style');
            styles.setAttribute('type', 'text/css');
            styles.setAttribute('id', id);
            var headRef = document.getElementsByTagName('head')[0];
            headRef.appendChild(styles);

            if (this.getInternetExplorerVersion() > 8) {
                styles.styleSheet.cssText = text;
            } else if (this.AddonBridge.IsBrowser("MSIE")) {
                try {
                    var rules = text.split("}");
                    var sheet = document.styleSheets[document.styleSheets.length - 1];
                    for (var i = 0; i < rules.length; i++) {
                        var rule = rules[i];
                        var parts = rule.split("{");
                        if (parts[0] == null || parts[1] == null)
                            continue;
                        sheet.addRule(parts[0], parts[1], 0);
                    }
                } catch (e) {
                    return false;
                }

            } else {
                var textNode = document.createTextNode(text);
                styles.appendChild(textNode);
            }

        return true;
    };
    
    RLLibrary.parseXMLFromString = function(xmlstring) {
        if (typeof window.DOMParser != "undefined") {
            return (new window.DOMParser()).parseFromString(xmlstring, "text/xml");
        } else if (typeof window.ActiveXObject != "undefined" && new window.ActiveXObject("Microsoft.XMLDOM")) {
            var xmlDoc = new window.ActiveXObject("Microsoft.XMLDOM");
            xmlDoc.async = "false";
            xmlDoc.loadXML(xmlstring);
            return xmlDoc;
        } else {
            throw new Error("No XML parser found");
        }
        return null;
    };
    
    RLLibrary.setSelectionRange = function(input, selectionStart, selectionEnd) {
        if (input.setSelectionRange) {
            input.setSelectionRange(selectionStart, selectionEnd);
        } else if (input.createTextRange) {
            var range = input.createTextRange();
            range.collapse(true);
            range.moveStart('character', selectionStart);
            range.moveEnd('character', selectionEnd);
            range.select();
        }
    };

    RLLibrary.getSelStart = function(o) {

        if (o.selectionStart)
            return o.selectionStart;

        var value = o.value || o.innerText;
        
        var r = document.selection != null ? document.selection.createRange().duplicate() : document.getSelection();
        r.moveEnd('textedit', value.length);
        if (r.text.length == 0) return value.length;

        return o.value.lastIndexOf(r.text);
    };

    RLLibrary.getTextWidth = function(text, styles) {
    
        var div = document.createElement("div");
        div.style.cssText = "position:absolute;" + styles;
        div.innerText = text;
        
        document.body.appendChild(div);
        var rect = div.getBoundingClientRect();
        var width = rect.right - rect.left;
        document.body.removeChild(div);
        
        return width;
    };
    
    RLLibrary.getTextHeight = function(text, styles) {
        var div = document.createElement("div");
        div.style.cssText = "position:absolute;" + styles;
        div.innerText = text;
        document.body.appendChild(div);

        var rect = div.getBoundingClientRect();
        var height = rect.bottom - rect.top;

        document.body.removeChild(div);
        return height;
    };

    RLLibrary.indexInArray = function(arr, el) {
        for (var i = 0; i < arr.length; i++) {
            if (arr[i] == el)
                return i;
        }
        return -1;
    };
    
    RLLibrary.getTextAttr = function(text, styles, attr) {

        var div = document.createElement("div");
        div.style.cssText = "position:absolute;" + styles;
        div.innerText = text;
        document.body.appendChild(div);

        var res = RLLibrary.getComputedStyle(div, null).getPropertyValue(attr);
        document.body.removeChild(div);

        return res;
    };

    RLLibrary.getElementOffsetX = function(element) {
        var x = 0;
        while (element != null && element.tagName.toLowerCase() != "body") {
            x += element.offsetLeft;
            //alert([x, element.offsetLeft, element.tagName])
            element = element.offsetParent;
        }
        return x;
    };
    
    RLLibrary.getElementOffsetY = function(element) {
        var y = 0;
        while (element != null) {
            y += element.offsetTop;
            element = element.offsetParent;
        }
        return y;
      };
      
      
    RLLibrary.mouseXFromEvent = function(evt) {
        if (evt.pageX) return evt.pageX;
        else if (evt.clientX)
            return evt.clientX + (document.documentElement.scrollLeft ?
                   document.documentElement.scrollLeft :
                   document.body.scrollLeft);
        else return null;
    };

    RLLibrary.mouseYFromEvent = function(evt) {
        if (evt.pageY) return evt.pageY;
        else if (evt.clientY)
            return evt.clientY + (document.documentElement.scrollTop ?
                   document.documentElement.scrollTop :
                   document.body.scrollTop);
        else return null;
    };
    
    RLLibrary.getComputedStyle = function(el, pseudo) {
        if (!window.getComputedStyle)
            window.getComputedStyle = function(el, pseudo) {
                this.el = el;
                this.getPropertyValue = function(prop) {
                    var re = /(\-([a-z]){1})/g;
                    if (prop == 'float') prop = 'styleFloat';
                    if (re.test(prop)) {
                        prop = prop.replace(re, function() {
                            return arguments[2].toUpperCase();
                        });
                    }
                    return el.currentStyle[prop] ? el.currentStyle[prop] : null;
                }
                return this;
            }

        return window.getComputedStyle(el, pseudo);
    };

    RLLibrary.AddonBridge = {
        _getClientName: function() {
            return RLLibrary._instanceName;
        },

        _getClientApiEventName: function() {
            return this._getClientName() + "-client-event";
        },

        _getAddonApiEventName: function() {
            return this._getClientName() + "-addon-event";
        },

        _getClientApiBridge: function() {
            return this._getClientName() + "-bridge";
        },

        sendMessage: function(message, callback, params) {

            if (this.IsBrowser("MSIE"))
                return this._sendIEMessage(message, callback, params);
            else if (this.IsBrowser("Firefox"))
                return this._sendFFMessage(message, callback, params);
            else if (this.IsBrowser("Chrome"))
                return this._sendChromeMessage(message, callback, params);
        },

        IsBrowser: function(name) {
            return navigator.userAgent.indexOf(name) != -1;
        },

        _bridgeElement: function() {
            if (!this.__bridgeElement) {
                this.__bridgeElement = document.createElement(this._getClientName() + "-page-api-bridge");
                document.documentElement.appendChild(this.__bridgeElement);
            }
            return this.__bridgeElement;
        },

        _processResponse: function(e) {
            var callback = e.target._callback;
            if (callback != null)
                callback(e.target.getAttribute("response"));
            e.target._callback == null;
        },

        _sendIEMessage: function(message, callback, params) {

            var doc = document;
            var newEvt = doc.createEventObject();
            newEvt.setAttribute("sourceElementID", this._getClientApiEventName());
            newEvt.setAttribute("message", message);
            newEvt.setAttribute("callback", callback);
            newEvt.setAttribute("params", (params == null) ? "" : params);

            var me = this;
            var handler = (function(sender) {
                return function(e) {
                    if (e.getAttribute("sourceElementID") != me._getAddonApiEventName())
                        return;

                    doc.detachEvent("onpropertychange", arguments.callee);

                    if (sender == null)
                        return;

                    sender(e.getAttribute("response"));
                    sender = null;
                }
            })(callback);
            doc.attachEvent("onpropertychange", handler);
            doc.fireEvent("onpropertychange", newEvt);
        },

        _sendFFMessage: function(message, callback, params) {
            var messageElement = document.createElement(this._getClientApiBridge());
            messageElement.setAttribute("message", message);
            messageElement.setAttribute("params", params);
            messageElement._callback = callback;
            this._bridgeElement().appendChild(messageElement);
            if (callback != null)
                messageElement.addEventListener("answerHandle", (function(me) {
                    return function(e) {
                        messageElement.removeEventListener("answerHandle", arguments.callee, false);
                        callback(e.target.getAttribute("response"));
                        me._bridgeElement().removeChild(messageElement);
                    }
                })(this), false, true);

            var e = document.createEvent("Events");
            e.initEvent(this._getClientApiEventName(), true, false);
            messageElement.dispatchEvent(e);
            if (callback == null)
                this._bridgeElement().removeChild(messageElement);
        },

        _sendChromeMessage: function(message, callback, params) {
            chrome.extension.sendRequest({ 'action': message, 'params': params }, callback);
        }

    };
    (function($) {
        var m = {
            '\b': '\\b',
            '\t': '\\t',
            '\n': '\\n',
            '\f': '\\f',
            '\r': '\\r',
            '"': '\\"',
            '\\': '\\\\'
        },
        s = {
            'array': function(x) {
                var a = ['['], b, f, i, l = x.length, v;
                for (i = 0; i < l; i += 1) {
                    v = x[i];
                    f = s[typeof v];
                    if (f) {
                        v = f(v);
                        if (typeof v == 'string') {
                            if (b) {
                                a[a.length] = ',';
                            }
                            a[a.length] = v;
                            b = true;
                        }
                    }
                }
                a[a.length] = ']';
                return a.join('');
            },
            'boolean': function(x) {
                return String(x);
            },
            'null': function(x) {
                return "null";
            },
            'number': function(x) {
                return isFinite(x) ? String(x) : 'null';
            },
            'object': function(x) {
                if (x) {
                    if (x instanceof Array) {
                        return s.array(x);
                    }
                    var a = ['{'], b, f, i, v;
                    for (i in x) {
                        v = x[i];
                        f = s[typeof v];
                        if (f) {
                            v = f(v);
                            if (typeof v == 'string') {
                                if (b) {
                                    a[a.length] = ',';
                                }
                                a.push(s.string(i), ':', v);
                                b = true;
                            }
                        }
                    }
                    a[a.length] = '}';
                    return a.join('');
                }
                return 'null';
            },
            'string': function(x) {
                if (/["\\\x00-\x1f]/.test(x)) {
                    x = x.replace(/([\x00-\x1f\\"])/g, function(a, b) {
                        var c = m[b];
                        if (c) {
                            return c;
                        }
                        c = b.charCodeAt();
                        return '\\u00' +
                            Math.floor(c / 16).toString(16) +
                            (c % 16).toString(16);
                    });
                }
                return '"' + x + '"';
            }
        };

        $.toJSON = function(v) {
            var f = isNaN(v) ? s[typeof v] : s['number'];
            if (f) return f(v);
        };

        $.parseJSON = function(v, safe) {
            if (safe === undefined) safe = $.parseJSON.safe;
            if (safe && !/^("(\\.|[^"\\\n\r])*?"|[,:{}\[\]0-9.\-+Eaeflnr-u \n\r\t])+?$/.test(v))
                return undefined;
            return eval('(' + v + ')');
        };

        $.parseJSON.safe = false;

    })(RLLibrary);
    
    return RLLibrary;
}

window.RLLibrary = CRLLibrary;

