const {
	classes: Cc,
	interfaces: Ci,
	results: Cr,
	utils: Cu
} = Components;

const EXTENSION_PATH = Cc["@mozilla.org/network/io-service;1"]
							.getService(Ci.nsIIOService)
							.newFileURI(__LOCATION__.parent.parent).spec;

Cu.import("resource://gre/modules/XPCOMUtils.jsm");


function CustomBarProtocol() {
	this._providers = [];
	this._observerService.addObserver(this, "quit-application", false);
    let c = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
    this._currentUserAgentLocale = c.getComplexValue("general.useragent.locale", Ci.nsISupportsString).data;

};

CustomBarProtocol.prototype = {

	get wrappedJSObject() {
		return this;
	},

	observe: function CBPH_observe(aSubject, aTopic, aData) {
		switch (aTopic) {
			case "quit-application":
				this._destroy();
				break;
		}
	},

	get scheme() {
		return this._scheme;
	},

	get protocolFlags() {
		return Ci.nsIProtocolHandler.URI_DANGEROUS_TO_LOAD |
				Ci.nsIProtocolHandler.URI_NOAUTH |
				Ci.nsIProtocolHandler.URI_IS_UI_RESOURCE |
				Ci.nsIProtocolHandler.URI_IS_LOCAL_FILE;
	},

	get defaultPort() {
		return -1;
	},

	allowPort: function CBPH_allowPort(port, scheme) {
		return false;
	},

	newURI: function CBPH_newURI(aSpec, aOriginalCharset, baseURI) {
	
		let uri = aSpec;

		let paths = uri.split("/");
		let url = "";
		for(let i = 2; i < paths.length; i++) {
			let entry = paths[i];
			if (entry == null)
				continue;
			url += "/" + entry;
		}

		let baseScheme =  (baseURI == null || baseURI.scheme != "file") ? "resource://smartsearchbox" : "chrome://smartsearchbox/content";
		let resultURL = baseScheme + url;

		if (resultURL.indexOf("/locale/") != -1) {
		    let localeResult = resultURL.replace("/locale/", "/locale/" + this._currentUserAgentLocale + "/");

		    let localeURI = this._ioService.newURI(localeResult, aOriginalCharset, null);
		    let channel = this._ioService.newChannelFromURI(localeURI);
		    try {
                channel.open();
                return localeURI;
            } catch (e) {
            }
	     }
		return this._ioService.newURI(resultURL, aOriginalCharset, null);
	},

	newChannel: function CBPH_newChannel(aURI) {
		
		let uri = aURI.path;

		if (uri.indexOf("#") != -1)
			uri = uri.substr(0, uri.indexOf("#"));

		if (uri.indexOf("?") != -1)
			uri = uri.substr(0, uri.indexOf("?"));

		let paths = uri.split("/");

		var file = __LOCATION__.parent.parent;
					
		file.append("chrome");
		file.append("content");

		for(let i = 0; i < paths.length; i++) {
			let entry = paths[i];
			if (entry == null)
				continue;
			file.append(entry);
		}
		
		let ios = Cc["@mozilla.org/network/io-service;1"]
							.getService(Components.interfaces.nsIIOService);
		let fileURI = ios.newFileURI(file);
		let channel = ios.newChannelFromURI(fileURI);
		channel.originalURI = aURI;
		return channel;
	},

	_scheme: "se-suggest",
	_ioService: Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService),
	_observerService: Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService),


	_destroy: function CBPH__destroy() {
		this._observerService.removeObserver(this, "quit-application");
	},
	
	classDescription: "Custom Slider protocol handler JS component for packages",
	classID: Components.ID("{48AF94B0-9DB3-44b2-B9B6-6692AB48CDEE}"),
	contractID: "@mozilla.org/network/protocol;1?name=se-suggest",
	QueryInterface: XPCOMUtils.generateQI([Ci.nsIProtocolHandler]),

	_xpcom_categories: [{
		category: "app-startup",
		service: true
	}]
};

if ("nsIContentSecurityPolicy" in Ci) {
	CustomBarProtocol.prototype.schemeIs = function (aScheme) {
		aScheme = aScheme.toLowerCase();
		return (aScheme == this._scheme) || (aScheme == "chrome");
	}
} else {
	CustomBarProtocol.prototype.schemeIs = function (aScheme) {
		aScheme = aScheme.toLowerCase();
		return (aScheme == this.scheme) || (aScheme == "chrome");
	}
}

if (XPCOMUtils.generateNSGetFactory)
  var NSGetFactory = XPCOMUtils.generateNSGetFactory([CustomBarProtocol]);
else
  var NSGetModule = XPCOMUtils.generateNSGetModule([CustomBarProtocol]);
